@@ -29,8 +29,9 @@
 ###
 
 """
-Add a description of the plugin (to be presented to the user inside the wizard)
-here.  This should describe *what* the plugin does.
+This plugin allows the user to create various aliases (known as "Akas", since
+Alias is the name of another plugin Aka is based on) to other commands or
+combinations of other commands (via nested commands).
 """
 
 import supybot
@@ -53,6 +54,7 @@ __url__ = '' # 'http://supybot.com/Members/yourname/Aka/download'
 from . import config
 from . import plugin
 from imp import reload
+from imp import reload
 # In case we're being reloaded.
 reload(config)
 reload(plugin)
