@@ -38,6 +38,7 @@ import supybot.utils as utils
 import supybot.ircdb as ircdb
 from supybot.commands import *
 import supybot.plugins as plugins
+import supybot.utils.minisix as minisix
 import supybot.ircutils as ircutils
 import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization
@@ -106,7 +107,7 @@ if sqlite3:
 
         def has_aka(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             return self.get_db(channel).cursor() \
@@ -122,7 +123,7 @@ if sqlite3:
 
         def get_alias(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             cursor = self.get_db(channel).cursor()
             cursor.execute("""SELECT alias FROM aliases
@@ -137,7 +138,7 @@ if sqlite3:
             name = callbacks.canonicalName(name, preserve_spaces=True)
             if self.has_aka(channel, name):
                 raise AkaError(_('This Aka already exists.'))
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 if isinstance(name, str):
                     name = name.decode('utf8')
                 if isinstance(alias, str):
@@ -150,7 +151,7 @@ if sqlite3:
 
         def remove_aka(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             db.cursor().execute('DELETE FROM aliases WHERE name = ?', (name,))
@@ -158,7 +159,7 @@ if sqlite3:
 
         def lock_aka(self, channel, name, by):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             cursor = db.cursor().execute("""UPDATE aliases
@@ -170,7 +171,7 @@ if sqlite3:
 
         def unlock_aka(self, channel, name, by):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             cursor = db.cursor()
@@ -182,7 +183,7 @@ if sqlite3:
 
         def get_aka_lock(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             cursor = self.get_db(channel).cursor()
             cursor.execute("""SELECT locked, locked_by, locked_at
@@ -237,7 +238,7 @@ elif sqlalchemy:
 
         def has_aka(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             count = self.get_db(channel).query(SQLAlchemyAlias) \
                     .filter(SQLAlchemyAlias.name == name) \
@@ -249,7 +250,7 @@ elif sqlalchemy:
 
         def get_alias(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             try:
                 return self.get_db(channel).query(SQLAlchemyAlias.alias) \
@@ -261,7 +262,7 @@ elif sqlalchemy:
             name = callbacks.canonicalName(name, preserve_spaces=True)
             if self.has_aka(channel, name):
                 raise AkaError(_('This Aka already exists.'))
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 if isinstance(name, str):
                     name = name.decode('utf8')
                 if isinstance(alias, str):
@@ -272,7 +273,7 @@ elif sqlalchemy:
 
         def remove_aka(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             db.query(SQLAlchemyAlias).filter(SQLAlchemyAlias.name == name).delete()
@@ -280,7 +281,7 @@ elif sqlalchemy:
 
         def lock_aka(self, channel, name, by):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             try:
@@ -297,7 +298,7 @@ elif sqlalchemy:
 
         def unlock_aka(self, channel, name, by):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             db = self.get_db(channel)
             try:
@@ -314,7 +315,7 @@ elif sqlalchemy:
 
         def get_aka_lock(self, channel, name):
             name = callbacks.canonicalName(name, preserve_spaces=True)
-            if sys.version_info[0] < 3 and isinstance(name, str):
+            if minisix.PY2 and isinstance(name, str):
                 name = name.decode('utf8')
             try:
                 return self.get_db(channel) \
@@ -365,7 +366,15 @@ def findBiggestAt(alias):
     else:
         return 0
 
-AkaDB = plugins.DB('Aka', available_db)
+if 'sqlite3' in conf.supybot.databases() and 'sqlite3' in available_db:
+    AkaDB = SQLiteAkaDB
+elif 'sqlalchemy' in conf.supybot.databases() and 'sqlalchemy' in available_db:
+    log.warning('Aka\'s only enabled database engine is SQLAlchemy, which '
+            'is deprecated. Please consider adding \'sqlite3\' to '
+            'supybot.databases (and/or install sqlite3).')
+    AkaDB = SqlAlchemyAkaDB
+else:
+    raise plugins.NoSuitableDatabase(['sqlite3', 'sqlalchemy'])
 
 class Aka(callbacks.Plugin):
     """Aka is the improved version of the Alias plugin. It stores akas outside
@@ -376,7 +385,9 @@ class Aka(callbacks.Plugin):
     def __init__(self, irc):
         self.__parent = super(Aka, self)
         self.__parent.__init__(irc)
-        self._db = AkaDB()
+        # "sqlalchemy" is only for backward compatibility
+        filename = conf.supybot.directories.data.dirize('Aka.sqlalchemy.db')
+        self._db = AkaDB(filename)
 
     def isCommandMethod(self, name):
         args = name.split(' ')
@@ -387,7 +398,7 @@ class Aka(callbacks.Plugin):
             for cb in dynamic.irc.callbacks: # including this plugin
                 if cb.isCommandMethod(' '.join(args[0:-1])):
                     return False
-        if sys.version_info[0] < 3 and isinstance(name, str):
+        if minisix.PY2 and isinstance(name, str):
             name = name.decode('utf8')
         channel = dynamic.channel or 'global'
         return self._db.has_aka(channel, name) or \
@@ -413,7 +424,7 @@ class Aka(callbacks.Plugin):
             if ret:
                 return [first] + ret
         max_length = self.registryValue('maximumWordsInName')
-        for i in xrange(1, min(len(args)+1, max_length)):
+        for i in range(1, min(len(args)+1, max_length)):
             if self.isCommandMethod(callbacks.formatCommand(args[0:i])):
                 return args[0:i]
         return []
@@ -695,7 +706,8 @@ class Aka(callbacks.Plugin):
         if alias_plugin is None:
             irc.error(_('Alias plugin is not loaded.'), Raise=True)
         errors = {}
-        for (name, (command, locked, func)) in alias_plugin.aliases.items():
+        for (name, (command, locked, func)) in \
+                list(alias_plugin.aliases.items()):
             try:
                 self._add_aka('global', name, command)
             except AkaError as e:
@@ -711,38 +723,55 @@ class Aka(callbacks.Plugin):
     importaliasdatabase = wrap(importaliasdatabase, ['owner'])
 
     def list(self, irc, msg, args, optlist):
-        """[--channel <#channel>] [--keys]
+        """[--channel <#channel>] [--keys] [--unlocked|--locked]
 
         Lists all Akas defined for <channel>. If <channel> is not specified,
         lists all global Akas. If --keys is given, lists only the Aka names
         and not their commands."""
         channel = 'global'
+        filterlocked = filterunlocked = False
         for (option, arg) in optlist:
             if option == 'channel':
                 if not ircutils.isChannel(arg):
                     irc.error(_('%r is not a valid channel.') % arg,
                             Raise=True)
                 channel = arg
+            if option == 'locked':
+                filterlocked = True
+            if option == 'unlocked':
+                filterunlocked = True
         aka_list = self._db.get_aka_list(channel)
+        if filterlocked and filterunlocked:
+            irc.error(_('--locked and --unlocked are incompatible options.'), Raise=True)
+        elif filterlocked:
+            aka_list = [aka for aka in aka_list if
+                        self._db.get_aka_lock(channel, aka)[0]]
+        elif filterunlocked:
+            aka_list = [aka for aka in aka_list if not
+                        self._db.get_aka_lock(channel, aka)[0]]
         if aka_list:
             if 'keys' in dict(optlist):
                 # Strange, aka_list is a list of one length tuples
                 s = [k[0] for k in aka_list]
+                oneToOne = True
             else:
                 aka_values = [self._db.get_alias(channel, aka) for aka in
                               aka_list]
                 s = ('{0}: "{1}"'.format(ircutils.bold(k), v) for (k, v) in
                     zip(aka_list, aka_values))
-            irc.replies(s)
+                oneToOne = None
+            irc.replies(s, oneToOne=oneToOne)
         else:
             irc.error(_("No Akas found."))
-    list = wrap(list, [getopts({'channel': 'channel', 'keys': ''})])
+    list = wrap(list, [getopts({'channel': 'channel', 'keys': '', 'locked': '',
+                                'unlocked': ''})])
 
     def search(self, irc, msg, args, optlist, query):
         """[--channel <#channel>] <query>
 
         Searches Akas defined for <channel>. If <channel> is not specified,
         searches all global Akas."""
+        query = callbacks.canonicalName(query, preserve_spaces=True)
         channel = 'global'
         for (option, arg) in optlist:
             if option == 'channel':
@@ -751,12 +780,13 @@ class Aka(callbacks.Plugin):
                             Raise=True)
                 channel = arg
         aka_list = self._db.get_aka_list(channel)
-        aka_list = [k[0] for k in aka_list]
-        if aka_list:
-            matching = [aka for aka in aka_list if query in aka]
+        aka_list = [callbacks.canonicalName(k[0], preserve_spaces=True)
+                    for k in aka_list]
+        matching = [aka for aka in aka_list if query in aka]
+        if matching:
             irc.replies(matching)
         else:
-            irc.error(_("No matching Akas found."))
+            irc.error(_("No matching Akas were found."))
     search = wrap(search, [getopts({'channel': 'channel'}), 'text'])
 
 Class = Aka
