@@ -31,14 +31,12 @@
 import os
 import sys
 import time
-if sys.version_info[0] < 3:
-    from io import open
-from cStringIO import StringIO
 
 import supybot.conf as conf
 import supybot.world as world
 import supybot.ircdb as ircdb
 import supybot.irclib as irclib
+import supybot.utils.minisix as minisix
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
 import supybot.registry as registry
@@ -46,6 +44,9 @@ import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('ChannelLogger')
 
+if minisix.PY2:
+    from io import open
+
 class FakeLog(object):
     def flush(self):
         return
@@ -60,8 +61,6 @@ class ChannelLogger(callbacks.Plugin):
     def __init__(self, irc):
         self.__parent = super(ChannelLogger, self)
         self.__parent.__init__(irc)
-        self.lastMsgs = {}
-        self.lastStates = {}
         self.logs = {}
         self.flusher = self.flush
         world.flushers.append(self.flusher)
@@ -71,30 +70,14 @@ class ChannelLogger(callbacks.Plugin):
             log.close()
         world.flushers = [x for x in world.flushers if x is not self.flusher]
 
-    def __call__(self, irc, msg):
-        try:
-            # I don't know why I put this in, but it doesn't work, because it
-            # doesn't call doNick or doQuit.
-            # if msg.args and irc.isChannel(msg.args[0]):
-            self.__parent.__call__(irc, msg)
-            if irc in self.lastMsgs:
-                if irc not in self.lastStates:
-                    self.lastStates[irc] = irc.state.copy()
-                self.lastStates[irc].addMsg(irc, self.lastMsgs[irc])
-        finally:
-            # We must make sure this always gets updated.
-            self.lastMsgs[irc] = msg
-
     def reset(self):
         for log in self._logs():
             log.close()
         self.logs.clear()
-        self.lastMsgs.clear()
-        self.lastStates.clear()
 
     def _logs(self):
-        for logs in self.logs.itervalues():
-            for log in logs.itervalues():
+        for logs in self.logs.values():
+            for log in logs.values():
                 yield log
 
     def flush(self):
@@ -133,7 +116,7 @@ class ChannelLogger(callbacks.Plugin):
 
     def checkLogNames(self):
         for (irc, logs) in self.logs.items():
-            for (channel, log) in logs.items():
+            for (channel, log) in list(logs.items()):
                 if self.registryValue('rotateLogs', channel):
                     name = self.getLogName(channel)
                     if name != os.path.basename(log.name):
@@ -164,7 +147,7 @@ class ChannelLogger(callbacks.Plugin):
         format = conf.supybot.log.timestampFormat()
         if format:
             string = time.strftime(format) + '  '
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 string = string.decode('utf8', 'ignore')
             log.write(string)
 
@@ -181,7 +164,7 @@ class ChannelLogger(callbacks.Plugin):
             self.timestamp(log)
         if self.registryValue('stripFormatting', channel):
             s = ircutils.stripFormatting(s)
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             s = s.decode('utf8', 'ignore')
         log.write(s)
         if self.registryValue('flushImmediately'):
@@ -221,10 +204,16 @@ class ChannelLogger(callbacks.Plugin):
     def doNick(self, irc, msg):
         oldNick = msg.nick
         newNick = msg.args[0]
-        for (channel, c) in irc.state.channels.items():
-            if newNick in c.users:
-                self.doLog(irc, channel,
-                           '*** %s is now known as %s\n', oldNick, newNick)
+        for channel in msg.tagged('channels'):
+            self.doLog(irc, channel,
+                       '*** %s is now known as %s\n', oldNick, newNick)
+
+    def doInvite(self, irc, msg):
+        (target, channel) = msg.args
+        self.doLog(irc, channel,
+                   '*** %s <%s> invited %s to %s\n',
+                   msg.nick, msg.prefix, target, channel)
+
     def doJoin(self, irc, msg):
         for channel in msg.args[0].split(','):
             if(self.registryValue('showJoinParts', channel)):
@@ -277,16 +266,11 @@ class ChannelLogger(callbacks.Plugin):
             reason = " (%s)" % msg.args[0]
         else:
             reason = ""
-        if not isinstance(irc, irclib.Irc):
-            irc = irc.getRealIrc()
-        if irc not in self.lastStates:
-            return
-        for (channel, chan) in self.lastStates[irc].channels.items():
+        for channel in msg.tagged('channels'):
             if(self.registryValue('showJoinParts', channel)):
-                if msg.nick in chan.users:
-                    self.doLog(irc, channel,
-                               '*** %s <%s> has quit IRC%s\n',
-                               msg.nick, msg.prefix, reason)
+                self.doLog(irc, channel,
+                           '*** %s <%s> has quit IRC%s\n',
+                           msg.nick, msg.prefix, reason)
 
     def outFilter(self, irc, msg):
         # Gotta catch my own messages *somehow* :)
