@@ -10,9 +10,10 @@
 #
 #****************************************************************************
 
-import re, copy, sys, os.path, StringIO
+import re, copy, sys, os.path
 
 import supybot.conf as conf
+import supybot.utils.minisix as minisix
 import supybot.registry as registry
 
 unitData = \
@@ -1059,7 +1060,7 @@ class UnitData(dict):
         types = []
         typeUnits = {}
         try:
-            f = StringIO.StringIO(unitData)
+            f = minisix.io.StringIO(unitData)
             lines = f.readlines()
             f.close()
         except IOError:
@@ -1083,7 +1084,7 @@ class UnitData(dict):
         for unit in units:
             self[unit.name.replace(' ', '')] = unit
             typeUnits[unit.typeName].append(unit.name)
-        self.sortedKeys = self.keys()
+        self.sortedKeys = list(self.keys())
         self.sortedKeys.sort()
 
         if len(self.sortedKeys) < len(units):
