@@ -44,6 +44,7 @@ import supybot.conf as conf
 import supybot.utils as utils
 import supybot.world as world
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
 import supybot.registry as registry
@@ -51,6 +52,11 @@ import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('RSS')
 
+if minisix.PY2:
+    from urllib2 import ProxyHandler
+else:
+    from urllib.request import ProxyHandler
+
 def get_feedName(irc, msg, args, state):
     if ircutils.isChannel(args[0]):
         state.errorInvalid('feed name', args[0], 'must not be channel names.')
@@ -75,6 +81,9 @@ def only_one_at_once(f):
             lock[0] = False
     return newf
 
+class InvalidFeedUrl(ValueError):
+    pass
+
 class Feed:
     __slots__ = ('url', 'name', 'data', 'last_update', 'entries',
             'etag', 'modified', 'initial',
@@ -83,7 +92,8 @@ class Feed:
             plugin_is_loading=False, announced=None):
         assert name, name
         if not url:
-            assert utils.web.httpUrlRe.match(name), name
+            if not utils.web.httpUrlRe.match(name):
+                raise InvalidFeedUrl(name)
             url = name
         self.name = name
         self.url = url
@@ -118,19 +128,31 @@ class Feed:
         f = types.MethodType(f, plugin)
         return f
 
+_sort_parameters = {
+        'oldestFirst':   (('published_parsed', 'updated_parsed'), False),
+        'newestFirst':   (('published_parsed', 'updated_parsed'), True),
+        'outdatedFirst': (('updated_parsed', 'published_parsed'), False),
+        'updatedFirst':  (('updated_parsed', 'published_parsed'), True),
+        }
+def _sort_arguments(order):
+    (fields, reverse) = _sort_parameters[order]
+    def key(entry):
+        for field in fields:
+            if field in entry:
+                return entry[field]
+        raise KeyError('No date field in entry.')
+    return (key, reverse)
+
 def sort_feed_items(items, order):
     """Return feed items, sorted according to sortFeedItems."""
-    if order not in ['oldestFirst', 'newestFirst']:
+    if order == 'asInFeed':
         return items
-    if order == 'oldestFirst':
-        reverse = False
-    if order == 'newestFirst':
-        reverse = True
+    (key, reverse) = _sort_arguments(order)
     try:
-        sitems = sorted(items, key=lambda i: i['published'], reverse=reverse)
+        sitems = sorted(items, key=key, reverse=reverse)
     except KeyError:
         # feedparser normalizes required timestamp fields in ATOM and RSS
-        # to the "published" field. Feeds missing it are unsortable by date.
+        # to the "published"/"updated" fields. Feeds missing it are unsortable by date.
         return items
     return sitems
 
@@ -167,7 +189,11 @@ class RSS(callbacks.Plugin):
             except registry.NonExistentRegistryEntry:
                 self.log.warning('%s is not a registered feed, removing.',name)
                 continue
-            self.register_feed(name, url, True, True, announced.get(name, []))
+            try:
+                self.register_feed(name, url, True, True, announced.get(name, []))
+            except InvalidFeedUrl:
+                self.log.error('%s is not a valid feed, removing.', name)
+                continue
         world.flushers.append(self._flush)
 
     def die(self):
@@ -208,6 +234,10 @@ class RSS(callbacks.Plugin):
         conf.registerChannelValue(feed_group, 'announceFormat',
                 registry.String('', _("""Feed-specific announce format.
                 Defaults to supybot.plugins.RSS.announceFormat if empty.""")))
+        conf.registerGlobalValue(feed_group, 'waitPeriod',
+                registry.NonNegativeInteger(0, _("""If set to a non-zero
+                value, overrides supybot.plugins.RSS.waitPeriod for this
+                particular feed.""")))
 
     def register_feed(self, name, url, initial,
             plugin_is_loading, announced=[]):
@@ -252,16 +282,27 @@ class RSS(callbacks.Plugin):
 
     def is_expired(self, feed):
         assert feed
-        event_horizon = time.time() - self.registryValue('waitPeriod')
+        period = self.registryValue('waitPeriod')
+        if feed.name != feed.url: # Named feed
+            specific_period = self.registryValue('feeds.%s.waitPeriod' % feed.name)
+            if specific_period:
+                period = specific_period
+        event_horizon = time.time() - period
         return feed.last_update < event_horizon
 
     ###############
     # Feed fetching
 
     def update_feed(self, feed):
+        handlers = []
+        if utils.web.proxy():
+            handlers.append(ProxyHandler(
+                {'http': utils.force(utils.web.proxy())}))
+            handlers.append(ProxyHandler(
+                {'https': utils.force(utils.web.proxy())}))
         with feed.lock:
             d = feedparser.parse(feed.url, etag=feed.etag,
-                    modified=feed.modified)
+                    modified=feed.modified, handlers=handlers)
             if 'status' not in d or d.status != 304: # Not modified
                 if 'etag' in d:
                     feed.etag = d.etag
@@ -294,7 +335,7 @@ class RSS(callbacks.Plugin):
         for name in announced_feeds:
             feed = self.get_feed(name)
             if not feed:
-                self.log.warning('Feed %s is announced but does not exist.' %
+                self.log.warning('Feed %s is announced but does not exist.',
                         name)
                 continue
             self.update_feed_if_needed(feed)
@@ -343,6 +384,15 @@ class RSS(callbacks.Plugin):
     def should_send_entry(self, channel, entry):
         whitelist = self.registryValue('keywordWhitelist', channel)
         blacklist = self.registryValue('keywordBlacklist', channel)
+
+        # fix shadowing by "from supybot.commands import *"
+        try:
+            all = __builtins__.all
+            any = __builtins__.any
+        except AttributeError:
+            all = __builtins__['all']
+            any = __builtins__['any']
+
         if whitelist:
             if all(kw not in entry.title and kw not in entry.description
                    for kw in whitelist):
@@ -353,6 +403,8 @@ class RSS(callbacks.Plugin):
                 return False
         return True
 
+    _normalize_entry = utils.str.multipleReplacer(
+            {'\r': ' ', '\n': ' ', '\x00': ''})
     def format_entry(self, channel, feed, entry, is_announce):
         key_name = 'announceFormat' if is_announce else 'format'
         if feed.name in self.registryValue('feeds'):
@@ -363,15 +415,20 @@ class RSS(callbacks.Plugin):
             template = self.registryValue(key_name, channel)
         date = entry.get('published_parsed')
         date = utils.str.timestamp(date)
-        return string.Template(template).safe_substitute(
+        s = string.Template(template).substitute(
+                entry,
                 feed_name=feed.name,
-                date=date,
-                **entry)
+                date=date)
+        return self._normalize_entry(s)
 
     def announce_entry(self, irc, channel, feed, entry):
         if self.should_send_entry(channel, entry):
             s = self.format_entry(channel, feed, entry, True)
-            irc.queueMsg(ircmsgs.privmsg(channel, s))
+            if self.registryValue('notice', channel):
+                m = ircmsgs.notice(channel, s)
+            else:
+                m = ircmsgs.privmsg(channel, s)
+            irc.queueMsg(m)
 
 
     ##########
@@ -428,7 +485,8 @@ class RSS(callbacks.Plugin):
             message isn't sent in the channel itself.
             """
             plugin = irc.getCallback('RSS')
-            invalid_feeds = [x for x in feeds if not plugin.get_feed(x)]
+            invalid_feeds = [x for x in feeds if not plugin.get_feed(x)
+                             and not utils.web.urlRe.match(x)]
             if invalid_feeds:
                 irc.error(format(_('These feeds are unknown: %L'),
                     invalid_feeds), Raise=True)
@@ -440,6 +498,10 @@ class RSS(callbacks.Plugin):
             irc.replySuccess()
             for name in feeds:
                 feed = plugin.get_feed(name)
+                if not feed:
+                    plugin.register_feed_config(name, name)
+                    plugin.register_feed(name, name, True, False)
+                    feed = plugin.get_feed(name)
                 plugin.announce_feed(feed, True)
         add = wrap(add, [('checkChannelCapability', 'op'),
                          many(first('url', 'feedName'))])
