@@ -37,7 +37,7 @@ import binascii
 
 import supybot.utils as utils
 from supybot.commands import *
-import supybot.commands as commands
+import supybot.utils.minisix as minisix
 import supybot.plugins as plugins
 import supybot.commands as commands
 import supybot.ircutils as ircutils
@@ -65,7 +65,7 @@ class String(callbacks.Plugin):
         Returns the character associated with the 8-bit value <number>
         """
         try:
-            irc.reply(chr(i))
+            irc.reply(chr(i), stripCtcp=False)
         except ValueError:
             irc.error(_('That number doesn\'t map to an 8-bit character.'))
     chr = wrap(chr, ['int'])
@@ -96,9 +96,9 @@ class String(callbacks.Plugin):
             text = codecs.getencoder('base64_codec')(text)[0].decode()
 
         # Change result into a string
-        if sys.version_info[0] < 3 and isinstance(text, unicode):
+        if minisix.PY2 and isinstance(text, unicode):
             text = text.encode('utf-8')
-        elif sys.version_info[0] >= 3 and isinstance(text, bytes):
+        elif minisix.PY3 and isinstance(text, bytes):
             text = text.decode()
 
         if encoding in ('base64', 'base64_codec'):
@@ -129,7 +129,7 @@ class String(callbacks.Plugin):
             decoder = codecs.getdecoder(encoding)
         except LookupError:
             irc.errorInvalid(_('encoding'), encoding)
-        if sys.version_info[0] >= 3 and not isinstance(text, bytes):
+        if minisix.PY3 and not isinstance(text, bytes):
             text = text.encode()
         try:
             text = decoder(text)[0]
@@ -140,9 +140,9 @@ class String(callbacks.Plugin):
             return
 
         # Change result into a string
-        if sys.version_info[0] < 3 and isinstance(text, unicode):
+        if minisix.PY2 and isinstance(text, unicode):
             text = text.encode('utf-8')
-        elif sys.version_info[0] >= 3 and isinstance(text, bytes):
+        elif minisix.PY3 and isinstance(text, bytes):
             try:
                 text = text.decode()
             except UnicodeDecodeError:
@@ -191,7 +191,7 @@ class String(callbacks.Plugin):
     len = wrap(len, ['text'])
 
     @internationalizeDocstring
-    def re(self, irc, msg, args, ff, text):
+    def re(self, irc, msg, args, f, text):
         """<regexp> <text>
 
         If <regexp> is of the form m/regexp/flags, returns the portion of
@@ -199,10 +199,6 @@ class String(callbacks.Plugin):
         s/regexp/replacement/flags, returns the result of applying such a
         regexp to <text>.
         """
-        if isinstance(ff, (types.FunctionType, types.MethodType)):
-            f = ff
-        else:
-            f = lambda s: ff.search(s) and ff.search(s).group(0) or ''
         if f('') and len(f(' ')) > len(f(''))+1: # Matches the empty string.
             s = _('You probably don\'t want to match the empty string.')
             irc.error(s)
@@ -210,12 +206,14 @@ class String(callbacks.Plugin):
             t = self.registryValue('re.timeout')
             try:
                 v = process(f, text, timeout=t, pn=self.name(), cn='re')
+                if isinstance(v, list):
+                    v = format('%L', v)
                 irc.reply(v)
             except commands.ProcessTimeoutError as e:
                 irc.error("ProcessTimeoutError: %s" % (e,))
             except re.error as e:
                 irc.error(e.args[0])
-    re = thread(wrap(re, [first('regexpMatcher', 'regexpReplacer'),
+    re = thread(wrap(re, [first('regexpMatcherMany', 'regexpReplacer'),
                    'text']))
 
     def xor(self, irc, msg, args, password, text):
