@@ -33,7 +33,7 @@ import supybot.registry as registry
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Unix')
 
-import plugin
+from . import plugin
 
 progstats = plugin.progstats
 
@@ -52,6 +52,20 @@ def configure(advanced):
           default=True):
         conf.supybot.commands.disabled().add('Unix.progstats')
 
+class NonOptionString(registry.String):
+    errormsg = _('Value must be a string not starting with a dash (-), not %r.')
+    def __init__(self, *args, **kwargs):
+        self.__parent = super(NonOptionString, self)
+        self.__parent.__init__(*args, **kwargs)
+    def setValue(self, v):
+        if v.startswith('-'):
+            self.error(v)
+        else:
+            self.__parent.setValue(v)
+
+class SpaceSeparatedListOfNonOptionStrings(registry.SpaceSeparatedListOfStrings):
+    Value = NonOptionString
+
 
 Unix = conf.registerPlugin('Unix')
 conf.registerGroup(Unix, 'fortune')
@@ -61,7 +75,7 @@ conf.registerGlobalValue(Unix.fortune, 'command',
 conf.registerChannelValue(Unix.fortune, 'short',
     registry.Boolean(True, _("""Determines whether only short fortunes will be
     used if possible.  This sends the -s option to the fortune program.""")))
-conf.registerGlobalValue(Unix.fortune, 'equal',
+conf.registerChannelValue(Unix.fortune, 'equal',
     registry.Boolean(True, _("""Determines whether fortune will give equal
     weight to the different fortune databases.  If false, then larger
     databases will be given more weight.  This sends the -e option to the
@@ -70,8 +84,8 @@ conf.registerChannelValue(Unix.fortune, 'offensive',
     registry.Boolean(False, _("""Determines whether fortune will retrieve
     offensive fortunes along with the normal fortunes.  This sends the -a
     option to the fortune program.""")))
-conf.registerGlobalValue(Unix.fortune, 'files',
-    registry.SpaceSeparatedListOfStrings([], _("""Determines what specific file
+conf.registerChannelValue(Unix.fortune, 'files',
+    SpaceSeparatedListOfNonOptionStrings([], _("""Determines what specific file
     (if any) will be used with the fortune command; if none is given, the
     system-wide default will be used.  Do note that this fortune file must be
     placed with the rest of your system's fortune files.""")))
