@@ -34,26 +34,18 @@ This module contains the basic callbacks for handling PRIVMSGs.
 """
 
 import re
-import sys
 import copy
 import time
-import shlex
+from . import shlex
 import codecs
 import getopt
 import inspect
 
-if sys.version_info[0] < 3:
-    # cStringIO is buggy with Python 2.6 (
-    # see http://paste.progval.net/show/227/ )
-    # and it does not handle unicode objects in Python  2.x
-    from StringIO import StringIO
-else:
-    from cStringIO import StringIO
-
-from . import (conf, ircdb, irclib, ircmsgs, ircutils, log, registry, utils,
-        world)
+from . import (conf, ircdb, irclib, ircmsgs, ircutils, log, registry,
+        utils, world)
+from .utils import minisix
 from .utils.iter import any, all
-from .i18n import PluginInternationalization, internationalizeDocstring
+from .i18n import PluginInternationalization
 _ = PluginInternationalization()
 
 def _addressed(nick, msg, prefixChars=None, nicks=None,
@@ -151,9 +143,9 @@ def canonicalName(command, preserve_spaces=False):
     Currently, this makes everything lowercase and removes all dashes and
     underscores.
     """
-    if sys.version_info[0] < 3 and isinstance(command, unicode):
+    if minisix.PY2 and isinstance(command, unicode):
         command = command.encode('utf-8')
-    elif sys.version_info[0] >= 3 and isinstance(command, bytes):
+    elif minisix.PY3 and isinstance(command, bytes):
         command = command.decode()
     special = '\t-_'
     if not preserve_spaces:
@@ -165,7 +157,8 @@ def canonicalName(command, preserve_spaces=False):
     return ''.join([x for x in command if x not in special]).lower() + reAppend
 
 def reply(msg, s, prefixNick=None, private=None,
-          notice=None, to=None, action=None, error=False):
+          notice=None, to=None, action=None, error=False,
+          stripCtcp=True):
     msg.tag('repliedTo')
     # Ok, let's make the target:
     # XXX This isn't entirely right.  Consider to=#foo, private=True.
@@ -196,6 +189,8 @@ def reply(msg, s, prefixNick=None, private=None,
         prefixNick = False
     if to is None:
         to = msg.nick
+    if stripCtcp:
+        s = s.strip('\x01')
     # Ok, now let's make the payload:
     s = ircutils.safeArgument(s)
     if not s and not action:
@@ -294,7 +289,7 @@ class Tokenizer(object):
             # Whoever you are, if you make a single modification to this
             # code, TEST the code with Python 2 & 3, both with the unit
             # tests and on IRC with this: @echo "好"
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 try:
                     token = token.encode('utf8').decode('string_escape')
                     token = token.decode('utf8')
@@ -325,11 +320,10 @@ class Tokenizer(object):
                 ret.append(self._insideBrackets(lexer))
             else:
                 ret.append(self._handleToken(token))
-            firstToken = False
         return ret
 
     def tokenize(self, s):
-        lexer = shlex.shlex(StringIO(s))
+        lexer = shlex.shlex(minisix.io.StringIO(s))
         lexer.commenters = ''
         lexer.quotes = self.quotes
         lexer.separators = self.separators
@@ -379,7 +373,6 @@ def tokenize(s, channel=None):
         if conf.get(nested.pipeSyntax, channel): # No nesting, no pipe.
             pipe = True
     quotes = conf.get(conf.supybot.commands.quotes, channel)
-    start = time.time()
     try:
         ret = Tokenizer(brackets=brackets,pipe=pipe,quotes=quotes).tokenize(s)
         return ret
@@ -390,7 +383,8 @@ def formatCommand(command):
     return ' '.join(command)
 
 def checkCommandCapability(msg, cb, commandName):
-    assert isinstance(commandName, basestring), commandName
+    if not isinstance(commandName, minisix.string_types):
+        commandName = '.'.join(commandName)
     plugin = cb.name().lower()
     pluginCommand = '%s.%s' % (plugin, commandName)
     def checkCapability(capability):
@@ -459,24 +453,37 @@ class RichReplyMethods(object):
         s = self.__makeReply(v, s)
         return self.reply(s, **kwargs)
 
+    def _getTarget(self, to=None):
+        """Compute the target according to self.to, the provided to,
+        and self.private, and return it. Mainly used by reply methods."""
+        # FIXME: Don't set self.to.
+        # I still set it to be sure I don't introduce a regression,
+        # but it does not make sense for .reply() and .replies() to
+        # change the state of this Irc object.
+        if to is not None:
+            self.to = self.to or to
+        target = self.private and self.to or self.msg.args[0]
+        return target
+
     def replies(self, L, prefixer=None, joiner=None,
-                onlyPrefixFirst=False, to=None,
+                onlyPrefixFirst=False,
                 oneToOne=None, **kwargs):
         if prefixer is None:
             prefixer = ''
         if joiner is None:
             joiner = utils.str.commaAndify
-        if isinstance(prefixer, basestring):
+        if isinstance(prefixer, minisix.string_types):
             prefixer = prefixer.__add__
-        if isinstance(joiner, basestring):
+        if isinstance(joiner, minisix.string_types):
             joiner = joiner.join
+        to = self._getTarget(kwargs.get('to'))
         if oneToOne is None: # Can be True, False, or None
             if ircutils.isChannel(to):
                 oneToOne = conf.get(conf.supybot.reply.oneToOne, to)
             else:
                 oneToOne = conf.supybot.reply.oneToOne()
         if oneToOne:
-            return self.reply(prefixer(joiner(L)), to=to, **kwargs)
+            return self.reply(prefixer(joiner(L)), **kwargs)
         else:
             msg = None
             first = True
@@ -484,11 +491,11 @@ class RichReplyMethods(object):
                 if onlyPrefixFirst:
                     if first:
                         first = False
-                        msg = self.reply(prefixer(s), to=to, **kwargs)
+                        msg = self.reply(prefixer(s), **kwargs)
                     else:
-                        msg = self.reply(s, to=to, **kwargs)
+                        msg = self.reply(s, **kwargs)
                 else:
-                    msg = self.reply(prefixer(s), to=to, **kwargs)
+                    msg = self.reply(prefixer(s), **kwargs)
             return msg
 
     def noReply(self):
@@ -503,29 +510,23 @@ class RichReplyMethods(object):
     def errorNoCapability(self, capability, s='', **kwargs):
         if 'Raise' not in kwargs:
             kwargs['Raise'] = True
-        if isinstance(capability, basestring): # checkCommandCapability!
-            log.warning('Denying %s for lacking %q capability.',
-                        self.msg.prefix, capability)
-            # noCapability means "don't send a specific capability error
-            # message" not "don't send a capability error message at all", like
-            # one would think
-            if self._getConfig(conf.supybot.reply.error.noCapability) or \
-                capability in conf.supybot.capabilities.private():
-                v = self._getConfig(conf.supybot.replies.genericNoCapability)
-            else:
-                v = self._getConfig(conf.supybot.replies.noCapability)
-                try:
-                    v %= capability
-                except TypeError: # No %s in string
-                    pass
-            s = self.__makeReply(v, s)
-            if s:
-                return self._error(s, **kwargs)
-        else:
-            log.warning('Denying %s for some unspecified capability '
-                        '(or a default).', self.msg.prefix)
+        log.warning('Denying %s for lacking %q capability.',
+                    self.msg.prefix, capability)
+        # noCapability means "don't send a specific capability error
+        # message" not "don't send a capability error message at all", like
+        # one would think
+        if self._getConfig(conf.supybot.reply.error.noCapability) or \
+            capability in conf.supybot.capabilities.private():
             v = self._getConfig(conf.supybot.replies.genericNoCapability)
-            return self._error(self.__makeReply(v, s), **kwargs)
+        else:
+            v = self._getConfig(conf.supybot.replies.noCapability)
+            try:
+                v %= capability
+            except TypeError: # No %s in string
+                pass
+        s = self.__makeReply(v, s)
+        if s:
+            return self._error(s, **kwargs)
 
     def errorPossibleBug(self, s='', **kwargs):
         v = self._getConfig(conf.supybot.replies.possibleBug)
@@ -564,7 +565,9 @@ class RichReplyMethods(object):
             v = _('That\'s not a valid %s.') % what
         if 'Raise' not in kwargs:
             kwargs['Raise'] = True
-        return self._error(self.__makeReply(v, s), **kwargs)
+        if s:
+            v += ' ' + s
+        return self._error(v, **kwargs)
 
 _repr = repr
 
@@ -674,10 +677,10 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
         else:
             self.prefixNick = conf.supybot.reply.withNickPrefix()
 
-    def evalArgs(self):
+    def evalArgs(self, withClass=None):
         while self.counter < len(self.args):
             self.repliedTo = False
-            if isinstance(self.args[self.counter], basestring):
+            if isinstance(self.args[self.counter], minisix.string_types):
                 # If it's a string, just go to the next arg.  There is no
                 # evaluation to be done for strings.  If, at some point,
                 # we decided to, say, convert every string using
@@ -691,8 +694,9 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                 # evaluating its args, it will call our reply method, which
                 # will subsequently call this function again, and we'll
                 # pick up where we left off via self.counter.
-                self.__class__(self, self.msg,
-                               self.args[self.counter], nested=self.nested+1)
+                cls = withClass or self.__class__
+                cls(self, self.msg, self.args[self.counter],
+                        nested=self.nested+1)
                 # We have to return here because the new NestedCommandsIrcProxy
                 # might not have called our reply method instantly, since
                 # its command might be threaded.  So (obviously) we can't
@@ -700,7 +704,7 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                 return
         # Once all the list args are evaluated, we then evaluate our own
         # list of args, since we're assured that they're all strings now.
-        assert all(lambda x: isinstance(x, basestring), self.args)
+        assert all(lambda x: isinstance(x, minisix.string_types), self.args)
         self.finalEval()
 
     def _callInvalidCommands(self):
@@ -836,7 +840,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                 cb._callCommand(command, self, self.msg, args)
 
     def reply(self, s, noLengthCheck=False, prefixNick=None, action=None,
-              private=None, notice=None, to=None, msg=None, sendImmediately=False):
+              private=None, notice=None, to=None, msg=None,
+              sendImmediately=False, stripCtcp=True):
         """
         Keyword arguments:
 
@@ -876,12 +881,10 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
             self.notice = self.notice or notice
         if private is not None:
             self.private = self.private or private
-        if to is not None:
-            self.to = self.to or to
+        target = self._getTarget(to)
         # action=True implies noLengthCheck=True and prefixNick=False
         self.noLengthCheck=noLengthCheck or self.noLengthCheck or self.action
-        target = self.private and self.to or self.msg.args[0]
-        if not isinstance(s, basestring): # avoid trying to str() unicode
+        if not isinstance(s, minisix.string_types): # avoid trying to str() unicode
             s = str(s) # Allow non-string esses.
         if self.finalEvaled:
             try:
@@ -892,7 +895,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                                           action=self.action,
                                           private=self.private,
                                           prefixNick=self.prefixNick,
-                                          noLengthCheck=self.noLengthCheck)
+                                          noLengthCheck=self.noLengthCheck,
+                                          stripCtcp=stripCtcp)
                 elif self.noLengthCheck:
                     # noLengthCheck only matters to NestedCommandsIrcProxy, so
                     # it's not used here.  Just in case you were wondering.
@@ -900,7 +904,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                               notice=self.notice,
                               action=self.action,
                               private=self.private,
-                              prefixNick=self.prefixNick)
+                              prefixNick=self.prefixNick,
+                              stripCtcp=stripCtcp)
                     sendMsg(m)
                     return m
                 else:
@@ -920,18 +925,20 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                         log.warning('Truncating to %s bytes from %s bytes.',
                                     maximumLength, len(s))
                         s = s[:maximumLength]
-                    if len(s) < allowedLength or \
+                    s_too_long = len(s.encode()) < allowedLength \
+                            if minisix.PY3 else len(s) < allowedLength
+                    if s_too_long or \
                        not conf.get(conf.supybot.reply.mores, target):
                         # In case we're truncating, we add 20 to allowedLength,
                         # because our allowedLength is shortened for the
                         # "(XX more messages)" trailer.
-                        if sys.version_info[0] >= 3:
+                        if minisix.PY3:
                             appended = _('(XX more messages)').encode()
-                            s = s.encode()[:allowedLength+len(appended)]
+                            s = s.encode()[:allowedLength-len(appended)]
                             s = s.decode('utf8', 'ignore')
                         else:
                             appended = _('(XX more messages)')
-                            s = s[:allowedLength+len(appended)]
+                            s = s[:allowedLength-len(appended)]
                         # There's no need for action=self.action here because
                         # action implies noLengthCheck, which has already been
                         # handled.  Let's stick an assert in here just in case.
@@ -939,7 +946,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                         m = reply(msg, s, to=self.to,
                                   notice=self.notice,
                                   private=self.private,
-                                  prefixNick=self.prefixNick)
+                                  prefixNick=self.prefixNick,
+                                  stripCtcp=stripCtcp)
                         sendMsg(m)
                         return m
                     msgs = ircutils.wrap(s, allowedLength,
@@ -952,7 +960,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                         m = reply(msg, response, to=self.to,
                                   notice=self.notice,
                                   private=self.private,
-                                  prefixNick=self.prefixNick)
+                                  prefixNick=self.prefixNick,
+                                  stripCtcp=stripCtcp)
                         sendMsg(m)
                         # XXX We should somehow allow these to be returned, but
                         #     until someone complains, we'll be fine :)  We
@@ -985,7 +994,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                                             action=self.action,
                                             notice=self.notice,
                                             private=self.private,
-                                            prefixNick=self.prefixNick)
+                                            prefixNick=self.prefixNick,
+                                            stripCtcp=stripCtcp)
                     sendMsg(m)
                     return m
             finally:
@@ -1001,6 +1011,16 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
                 self.args[self.counter] = s
             self.evalArgs()
 
+    def replies(self, L, prefixer=None, joiner=None,
+                onlyPrefixFirst=False, to=None,
+                oneToOne=None, **kwargs):
+        if not self.finalEvaled and oneToOne is None:
+            oneToOne = True
+        return super(NestedCommandsIrcProxy, self).replies(L,
+                prefixer=prefixer, joiner=joiner,
+                onlyPrefixFirst=onlyPrefixFirst, to=to,
+                oneToOne=oneToOne, **kwargs)
+
     def error(self, s='', Raise=False, **kwargs):
         self.repliedTo = True
         if Raise:
@@ -1029,7 +1049,7 @@ class CommandThread(world.SupyThread):
     """
     def __init__(self, target=None, args=(), kwargs={}):
         self.command = args[0]
-        self.cb = target.im_self
+        self.cb = target.__self__
         threadName = 'Thread #%s (for %s.%s)' % (world.threadsSpawned,
                                                  self.cb.name(),
                                                  self.command)
@@ -1140,11 +1160,12 @@ class BasePlugin(object):
                 cb.log = log.getPluginLogger('%s.%s' % (self.name(),cb.name()))
         super(BasePlugin, self).__init__()
 
-class SynchronizedAndFirewalled(log.MetaFirewall, utils.python.Synchronized):
-    pass # Necessary for the metaclass compatibility issue.
+class MetaSynchronizedAndFirewalled(log.MetaFirewall, utils.python.MetaSynchronized):
+    pass
+SynchronizedAndFirewalled = MetaSynchronizedAndFirewalled(
+        'SynchronizedAndFirewalled', (), {})
 
-class Commands(BasePlugin):
-    __metaclass__ = SynchronizedAndFirewalled
+class Commands(BasePlugin, SynchronizedAndFirewalled):
     __synchronized__ = (
         '__call__',
         'callCommand',
@@ -1183,7 +1204,7 @@ class Commands(BasePlugin):
         if hasattr(self, name):
             method = getattr(self, name)
             if inspect.ismethod(method):
-                code = method.im_func.__code__
+                code = method.__func__.__code__
                 return inspect.getargs(code)[0] == self.commandArgs
             else:
                 return False
@@ -1192,7 +1213,7 @@ class Commands(BasePlugin):
 
     def isCommand(self, command):
         """Convenience, backwards-compatibility, semi-deprecated."""
-        if isinstance(command, basestring):
+        if isinstance(command, minisix.string_types):
             return self.isCommandMethod(command)
         else:
             # Since we're doing a little type dispatching here, let's not be
@@ -1218,7 +1239,7 @@ class Commands(BasePlugin):
     def getCommandMethod(self, command):
         """Gets the given command from this plugin."""
         #print '*** %s.getCommandMethod(%r)' % (self.name(), command)
-        assert not isinstance(command, basestring)
+        assert not isinstance(command, minisix.string_types)
         assert command == list(map(canonicalName, command))
         assert self.getCommand(command) == command
         for cb in self.cbs:
@@ -1230,7 +1251,7 @@ class Commands(BasePlugin):
         else:
             method = getattr(self, command[0])
             if inspect.ismethod(method):
-                code = method.im_func.__code__
+                code = method.__func__.__code__
                 if inspect.getargs(code)[0] == self.commandArgs:
                     return method
                 else:
@@ -1270,6 +1291,10 @@ class Commands(BasePlugin):
         # XXX I'm being extra-special-careful here, but we need to refactor
         #     this.
         try:
+            cap = checkCommandCapability(msg, self, command)
+            if cap:
+                irc.errorNoCapability(cap)
+                return
             for name in command:
                 cap = checkCommandCapability(msg, self, name)
                 if cap:
@@ -1289,7 +1314,7 @@ class Commands(BasePlugin):
             if 'command has no help.' in help:
                 # Note: this case will never happen, unless 'checkDoc' is set
                 # to False.
-                irc.error(_('Invalid arguments for %s.') % ' '.join(command))
+                irc.error(_('Invalid arguments for %s.') % formatCommand(command))
             else:
                 irc.reply(help)
         except (SyntaxError, Error) as e:
@@ -1341,7 +1366,11 @@ class PluginMixin(BasePlugin, irclib.IrcCallback):
     def __call__(self, irc, msg):
         irc = SimpleProxy(irc, msg)
         if msg.command == 'PRIVMSG':
-            if self.noIgnore or \
+            if hasattr(self.noIgnore, '__call__'):
+                noIgnore = self.noIgnore(irc, msg)
+            else:
+                noIgnore = self.noIgnore
+            if noIgnore or \
                not ircdb.checkIgnored(msg.prefix, msg.args[0]) or \
                not ircutils.isUserHostmask(msg.prefix):  # Some services impl.
                 self.__parent.__call__(irc, msg)
