@@ -34,6 +34,7 @@ import time
 import operator
 
 from . import conf, ircutils, log, registry, unpreserve, utils, world
+from .utils import minisix
 
 def isCapability(capability):
     return len(capability.split(None, 1)) == 1
@@ -303,7 +304,7 @@ class IrcUser(object):
     def addNick(self, network, nick):
         """Adds a nick to the user's registered nicks on the network."""
         global users
-        assert isinstance(network, basestring)
+        assert isinstance(network, minisix.string_types)
         assert ircutils.isNick(nick), 'got %s' % nick
         if users.getUserFromNick(network, nick) is not None:
             raise KeyError
@@ -314,7 +315,7 @@ class IrcUser(object):
 
     def removeNick(self, network, nick):
         """Removes a nick from the user's registered nicks on the network."""
-        assert isinstance(network, basestring)
+        assert isinstance(network, minisix.string_types)
         if nick not in self.nicks[network]:
             raise KeyError
         self.nicks[network].remove(nick)
@@ -351,8 +352,9 @@ class IrcUser(object):
         write('name %s' % self.name)
         write('ignore %s' % self.ignore)
         write('secure %s' % self.secure)
-        write('hashed %s' % self.hashed)
-        write('password %s' % self.password)
+        if self.password:
+            write('hashed %s' % self.hashed)
+            write('password %s' % self.password)
         for capability in self.capabilities:
             write('capability %s' % capability)
         for hostmask in self.hostmasks:
@@ -405,7 +407,7 @@ class IrcChannel(object):
         """Checks whether a given hostmask is banned by the channel banlist."""
         assert ircutils.isUserHostmask(hostmask), 'got %s' % hostmask
         now = time.time()
-        for (pattern, expiration) in self.bans.items():
+        for (pattern, expiration) in list(self.bans.items()):
             if now < expiration or not expiration:
                 if ircutils.hostmaskPatternEqual(pattern, hostmask):
                     return True
@@ -459,7 +461,7 @@ class IrcChannel(object):
         if self.checkBan(hostmask):
             return True
         now = time.time()
-        for (pattern, expiration) in self.ignores.items():
+        for (pattern, expiration) in list(self.ignores.items()):
             if now < expiration or not expiration:
                 if ircutils.hostmaskPatternEqual(pattern, hostmask):
                     return True
@@ -477,11 +479,11 @@ class IrcChannel(object):
         write('defaultAllow %s' % self.defaultAllow)
         for capability in self.capabilities:
             write('capability ' + capability)
-        bans = self.bans.items()
+        bans = list(self.bans.items())
         utils.sortBy(operator.itemgetter(1), bans)
         for (ban, expiration) in bans:
             write('ban %s %d' % (ban, expiration))
-        ignores = self.ignores.items()
+        ignores = list(self.ignores.items())
         utils.sortBy(operator.itemgetter(1), ignores)
         for (ignore, expiration) in ignores:
             write('ignore %s %d' % (ignore, expiration))
@@ -653,7 +655,7 @@ class UsersDictionary(utils.IterableMap):
         """Flushes the database to its file."""
         if not self.noFlush:
             if self.filename is not None:
-                L = self.users.items()
+                L = list(self.users.items())
                 L.sort()
                 fd = utils.file.AtomicFile(self.filename)
                 for (id, u) in L:
@@ -672,8 +674,8 @@ class UsersDictionary(utils.IterableMap):
             world.flushers.remove(self.flush)
         self.users.clear()
 
-    def iteritems(self):
-        return self.users.iteritems()
+    def items(self):
+        return self.users.items()
 
     def getUserId(self, s):
         """Returns the user ID of a given name or hostmask."""
@@ -682,12 +684,12 @@ class UsersDictionary(utils.IterableMap):
                 return self._hostmaskCache[s]
             except KeyError:
                 ids = {}
-                for (id, user) in self.users.iteritems():
+                for (id, user) in self.users.items():
                     x = user.checkHostmask(s)
                     if x:
                         ids[id] = x
                 if len(ids) == 1:
-                    id = ids.keys()[0]
+                    id = list(ids.keys())[0]
                     self._hostmaskCache[s] = id
                     try:
                         self._hostmaskCache[id].add(s)
@@ -699,7 +701,7 @@ class UsersDictionary(utils.IterableMap):
                 else:
                     log.error('Multiple matches found in user database.  '
                               'Removing the offending hostmasks.')
-                    for (id, hostmask) in ids.iteritems():
+                    for (id, hostmask) in ids.items():
                         log.error('Removing %q from user %s.', hostmask, id)
                         self.users[id].removeHostmask(hostmask)
                     raise DuplicateHostmask('Ids %r matched.' % ids)
@@ -777,7 +779,7 @@ class UsersDictionary(utils.IterableMap):
         except KeyError:
             pass
         for hostmask in user.hostmasks:
-            for (i, u) in self.iteritems():
+            for (i, u) in self.items():
                 if i == user.id:
                     continue
                 elif u.checkHostmask(hostmask):
@@ -802,7 +804,7 @@ class UsersDictionary(utils.IterableMap):
             del self._nameCache[self._nameCache[id]]
             del self._nameCache[id]
         if id in self._hostmaskCache:
-            for hostmask in self._hostmaskCache[id]:
+            for hostmask in list(self._hostmaskCache[id]):
                 del self._hostmaskCache[hostmask]
             del self._hostmaskCache[id]
         self.flush()
@@ -847,7 +849,7 @@ class ChannelsDictionary(utils.IterableMap):
         if not self.noFlush:
             if self.filename is not None:
                 fd = utils.file.AtomicFile(self.filename)
-                for (channel, c) in self.channels.iteritems():
+                for (channel, c) in self.channels.items():
                     fd.write('channel %s' % channel)
                     fd.write(os.linesep)
                     c.preserve(fd, indent='  ')
@@ -890,8 +892,8 @@ class ChannelsDictionary(utils.IterableMap):
         self.channels[channel] = ircChannel
         self.flush()
 
-    def iteritems(self):
-        return self.channels.iteritems()
+    def items(self):
+        return self.channels.items()
 
 
 class IgnoresDB(object):
@@ -912,7 +914,7 @@ class IgnoresDB(object):
                 else:
                     expiration = 0
                 self.add(hostmask, expiration)
-            except Exception as e:
+            except Exception:
                 log.error('Invalid line in ignores database: %q', line)
         fd.close()
 
@@ -949,7 +951,7 @@ class IgnoresDB(object):
 
     def checkIgnored(self, prefix):
         now = time.time()
-        for (hostmask, expiration) in self.hostmasks.items():
+        for (hostmask, expiration) in list(self.hostmasks.items()):
             if expiration and now > expiration:
                 del self.hostmasks[hostmask]
             else:
