@@ -65,7 +65,7 @@ def loadPluginModule(name, ignoreDeprecation=False):
         module = imp.load_module(name, *moduleInfo)
     except:
         sys.modules.pop(name, None)
-        keys = sys.modules.keys()
+        keys = list(sys.modules.keys())
         for key in keys:
             if key.startswith(name + '.'):
                 sys.modules.pop(key)
