@@ -33,9 +33,9 @@ from __future__ import print_function
 import os
 import sys
 import ast
-import time
-import types
 import textwrap
+import warnings
+import functools
 import traceback
 import collections
 
@@ -43,15 +43,25 @@ import collections
 from . import crypt
 from .str import format
 from .file import mktemp
-from .iter import imap
+from . import minisix
+from . import internationalization as _
+
+def warn_non_constant_time(f):
+    @functools.wraps(f)
+    def newf(*args, **kwargs):
+        # This method takes linear time whereas the subclass could probably
+        # do it in constant time.
+        warnings.warn('subclass of IterableMap does provide an efficient '
+                      'implementation of %s' % f.__name__,
+                      DeprecationWarning)
+        return f(*args, **kwargs)
+    return newf
 
-from supybot.i18n import PluginInternationalization
-_ = PluginInternationalization()
 
 def abbrev(strings, d=None):
     """Returns a dictionary mapping unambiguous abbreviations to full forms."""
     def eachSubstring(s):
-        for i in xrange(1, len(s)+1):
+        for i in range(1, len(s)+1):
             yield s[:i]
     if len(strings) != len(set(strings)):
         raise ValueError(
@@ -154,7 +164,7 @@ def saltHash(password, salt=None, hash='sha'):
         hasher = crypt.md5
     return '|'.join([salt, hasher((salt + password).encode('utf8')).hexdigest()])
 
-_astStr2 = ast.Str if sys.version_info[0] < 3 else ast.Bytes
+_astStr2 = ast.Str if minisix.PY2 else ast.Bytes
 def safeEval(s, namespace={'True': True, 'False': False, 'None': None}):
     """Evaluates s, safely.  Useful for turning strings into tuples/lists/etc.
     without unsafely using eval()."""
@@ -206,46 +216,35 @@ def exnToString(e):
         return e.__class__.__name__
 
 class IterableMap(object):
-    """Define .iteritems() in a class and subclass this to get the other iters.
+    """Define .items() in a class and subclass this to get the other iters.
     """
-    def iteritems(self):
-        if sys.version_info[0] >= 3 and hasattr(self, 'iteritems'):
+    def items(self):
+        if minisix.PY3 and hasattr(self, 'iteritems'):
             # For old plugins
-            return getattr(self, 'iteritems')() # avoid 2to3
+            return self.iteritems() # avoid 2to3
         else:
             raise NotImplementedError()
+    __iter__ = items
 
-    def iterkeys(self):
-        for (key, _) in self.iteritems():
+    def keys(self):
+        for (key, __) in self.items():
             yield key
 
-    def itervalues(self):
-        for (_, value) in self.iteritems():
+    def values(self):
+        for (__, value) in self.items():
             yield value
 
-    if sys.version_info[0] < 3:
-        # Our 2to3 fixers automatically rename iteritems/iterkeys/itervalues
-        # to items/keys/values
-        def items(self):
-            return list(self.iteritems())
-
-        def keys(self):
-            return list(self.iterkeys())
-
-        def values(self):
-            return list(self.itervalues())
-        __iter__ = iterkeys
-    else:
-        __iter__ = items
 
+    @warn_non_constant_time
     def __len__(self):
         ret = 0
-        for _ in self.iteritems():
+        for __ in self.items():
             ret += 1
         return ret
 
+    @warn_non_constant_time
     def __bool__(self):
-        for _ in self.iteritems():
+        for __ in self.items():
             return True
         return False
     __nonzero__ = __bool__
@@ -290,12 +289,15 @@ class InsensitivePreservingDict(collections.MutableMapping):
     def __len__(self):
         return len(self.data)
 
-    def iteritems(self):
-        return self.data.itervalues()
+    def items(self):
+        return self.data.values()
+
+    def items(self):
+        return self.data.values()
 
     def keys(self):
         L = []
-        for (k, _) in self.iteritems():
+        for (k, __) in self.items():
             L.append(k)
         return L
 
@@ -342,7 +344,7 @@ def stackTrace(frame=None, compact=True):
 def callTracer(fd=None, basename=True):
     if fd is None:
         fd = sys.stdout
-    def tracer(frame, event, _):
+    def tracer(frame, event, __):
         if event == 'call':
             code = frame.f_code
             lineno = frame.f_lineno
