@@ -92,6 +92,11 @@ class FunctionsTestCase(SupyTestCase):
         self.assertEqual(s[0], '\x02')
         self.assertEqual(s[-1], '\x02')
 
+    def testItalic(self):
+        s = ircutils.italic('foo')
+        self.assertEqual(s[0], '\x1d')
+        self.assertEqual(s[-1], '\x1d')
+
     def testUnderline(self):
         s = ircutils.underline('foo')
         self.assertEqual(s[0], '\x1f')
@@ -126,6 +131,9 @@ class FunctionsTestCase(SupyTestCase):
     def testStripBold(self):
         self.assertEqual(ircutils.stripBold(ircutils.bold('foo')), 'foo')
 
+    def testStripItalic(self):
+        self.assertEqual(ircutils.stripItalic(ircutils.italic('foo')), 'foo')
+
     def testStripColor(self):
         self.assertEqual(ircutils.stripColor('\x02bold\x0302,04foo\x03bar\x0f'),
                          '\x02boldfoobar\x0f')
@@ -149,6 +157,7 @@ class FunctionsTestCase(SupyTestCase):
 
     def testStripFormatting(self):
         self.assertEqual(ircutils.stripFormatting(ircutils.bold('foo')), 'foo')
+        self.assertEqual(ircutils.stripFormatting(ircutils.italic('foo')), 'foo')
         self.assertEqual(ircutils.stripFormatting(ircutils.reverse('foo')),
                          'foo')
         self.assertEqual(ircutils.stripFormatting(ircutils.underline('foo')),
@@ -210,6 +219,8 @@ class FunctionsTestCase(SupyTestCase):
         msg = ircmsgs.IrcMsg(':%s PRIVMSG #channel :stuff' % self.hostmask)
         class Irc(object):
             nick = 'bob'
+            network = 'testnet'
+
         irc = Irc()
 
         f = ircutils.standardSubstitute
@@ -382,6 +393,41 @@ class IrcStringTestCase(SupyTestCase):
         self.failUnless(s1 == s2)
         self.failIf(s1 != s2)
 
+class AuthenticateTestCase(SupyTestCase):
+    PAIRS = [
+            (b'', ['+']),
+            (b'foo'*150, [
+                'Zm9v'*100,
+                'Zm9v'*50
+                ]),
+            (b'foo'*200, [
+                'Zm9v'*100,
+                'Zm9v'*100,
+                '+'])
+            ]
+    def assertMessages(self, got, should):
+        got = list(got)
+        for (s1, s2) in zip(got, should):
+            self.assertEqual(s1, s2, (got, should))
+
+    def testGenerator(self):
+        for (decoded, encoded) in self.PAIRS:
+            self.assertMessages(
+                    ircutils.authenticate_generator(decoded),
+                    encoded)
+
+    def testDecoder(self):
+        for (decoded, encoded) in self.PAIRS:
+            decoder = ircutils.AuthenticateDecoder()
+            for chunk in encoded:
+                self.assertFalse(decoder.ready, (decoded, encoded))
+                decoder.feed(ircmsgs.IrcMsg(command='AUTHENTICATE',
+                    args=(chunk,)))
+            self.assertTrue(decoder.ready)
+            self.assertEqual(decoder.get(), decoded)
+
+
+
 
 # vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
 
