@@ -35,6 +35,7 @@ import time
 import pickle
 import supybot.utils as utils
 from supybot.utils.structures import *
+import supybot.utils.minisix as minisix
 
 if sys.version_info[0] >= 0:
     xrange = range
@@ -118,7 +119,7 @@ class GenTest(SupyTestCase):
             def __setitem__(self, key, value):
                 self.L.append((key, value))
 
-            def iteritems(self):
+            def items(self):
                 for (k, v) in self.L:
                     yield (k, v)
         AL = alist()
@@ -128,12 +129,12 @@ class GenTest(SupyTestCase):
         AL[3] = 4
         self.failUnless(AL)
         self.assertEqual(list(AL.items()), [(1, 2), (2, 3), (3, 4)])
-        self.assertEqual(list(AL.iteritems()), [(1, 2), (2, 3), (3, 4)])
+        self.assertEqual(list(AL.items()), [(1, 2), (2, 3), (3, 4)])
         self.assertEqual(list(AL.keys()), [1, 2, 3])
-        if sys.version_info[0] < 3:
-            self.assertEqual(list(AL.iterkeys()), [1, 2, 3])
-            self.assertEqual(AL.values(), [2, 3, 4])
-            self.assertEqual(list(AL.itervalues()), [2, 3, 4])
+        if minisix.PY2:
+            self.assertEqual(list(AL.keys()), [1, 2, 3])
+            self.assertEqual(list(AL.values()), [2, 3, 4])
+            self.assertEqual(list(AL.values()), [2, 3, 4])
         self.assertEqual(len(AL), 3)
 
     def testSortBy(self):
@@ -368,6 +369,7 @@ class StrTest(SupyTestCase):
         self.assertEqual(f('foo   bar'), 'foo bar')
         self.assertEqual(f('foo\nbar'), 'foo bar')
         self.assertEqual(f('foo\tbar'), 'foo bar')
+        self.assertEqual(f('foo\rbar'), 'foo bar')
 
     def testNItems(self):
         nItems = utils.str.nItems
@@ -539,6 +541,18 @@ class FormatTestCase(SupyTestCase):
                          'I have 3 kinds of fruit: '
                          'apples, oranges, and watermelon.')
 
+    def testPercentL(self):
+        self.assertIn(format('%L', set(['apples', 'oranges', 'watermelon'])), [
+                         'apples, oranges, and watermelon',
+                         'oranges, apples, and watermelon',
+                         'apples, watermelon, and oranges',
+                         'oranges, watermelon, and apples',
+                         'watermelon, apples, and oranges',
+                         'watermelon, oranges, and apples'])
+
+        self.assertEqual(format('%L',
+            (['apples', 'oranges', 'watermelon'], 'or')),
+            'apples, oranges, or watermelon')
 class RingBufferTestCase(SupyTestCase):
     def testInit(self):
         self.assertRaises(ValueError, RingBuffer, -1)
